@@ -6,13 +6,12 @@ from functools import partial
 
 import ipykernel
 from ipykernel.ipkernel import IPythonKernel
-from traitlets import Integer
-from traitlets import Type
+from traitlets import Integer, Type
+
+from ipyparallel.serialize import serialize_object, unpack_apply_message
+from ipyparallel.util import utcnow
 
 from .datapub import ZMQDataPublisher
-from ipyparallel.serialize import serialize_object
-from ipyparallel.serialize import unpack_apply_message
-from ipyparallel.util import utcnow
 
 
 class IPythonParallelKernel(IPythonKernel):
@@ -85,7 +84,7 @@ class IPythonParallelKernel(IPythonKernel):
 
         # if we have a delay, give messages this long to arrive on the queue
         # before we start accepting requests
-        asyncio.get_event_loop().call_later(
+        asyncio.get_running_loop().call_later(
             self.stop_on_error_timeout, schedule_stop_aborting
         )
 
@@ -212,7 +211,10 @@ class IPythonParallelKernel(IPythonKernel):
         except BaseException as e:
             # invoke IPython traceback formatting
             # this sends the 'error' message
-            shell.showtraceback()
+            try:
+                shell.showtraceback()
+            except Exception as tb_error:
+                self.log.error(f"Failed to show traceback for {e}: {tb_error}")
 
             try:
                 str_evalue = str(e)
